import { LotteryStats, LotteryStatsClass } from 'models/LotteryStats'
import { singleton } from 'zutils'
import { formatDate } from 'utils/utcdate.util'
import { DocumentType } from '@typegoose/typegoose'

const EXPIRE_TIME = 1000 * 60 * 60
@singleton
export class LotteryCache {
  map: Map<string, DocumentType<LotteryStatsClass>> = new Map()
  lastUsed: Map<string, number> = new Map()

  public async getData(user: string, activity: string) {
    const dateTag = formatDate(new Date())
    if (!this.map.has(user + dateTag)) {
      const record = await LotteryStats.insertOrUpdate({ user, activity, dateTag }, {})
      this.map.set(user + dateTag, record)
    }
    this.lastUsed.set(user + dateTag, Date.now())
    return this.map.get(user + dateTag)
  }
  public async flush() {
    for (let [key, record] of this.map.entries()) {
      // record.modifiedPaths()
      if (record.isModified()) {
        await record.save()
      }
      if (Date.now() - this.lastUsed.get(key) > EXPIRE_TIME) {
        this.map.delete(key)
        this.lastUsed.delete(key)
      }
    }
  }
}
