import { ZError, BaseController, router } from 'zutils'
import { SyncLocker } from 'common/SyncLocker'
import { FastifyRequest } from 'fastify'
import { ActivityItem } from 'models/ActivityItem'
import { TokenClaimHistory } from 'models/TokenClaimHistory'
import { queryStakeList } from 'services/chain.svr'
import { sign } from 'zutils/utils/chain.util'

export default class ChainController extends BaseController {
  // @router('get /api/stake/list')
  async stakeList(req) {
    const user = req.user
    const records = await queryStakeList(user.address.toLowerCase())
    const result = records.map(r => r.toJson())
    return result
  }

  // @router('post /api/lottery/claim_usdt')
  async preClaimUsdt(req: FastifyRequest) {
    await new SyncLocker().checkLock(req)
    let user = req.user
    if (user.address) {
      throw new ZError(11, 'no address')
    }
    const minClaimNum = +process.env.MINI_CLAIM_USDT
    const record = await ActivityItem.findOne({ user: user.id, activity: user.activity, item: 'usdt' })
    if (!record || record.amount < minClaimNum) {
      throw new ZError(10, 'no enough usdt')
    }
    const amount = record.amount
    record.amount = 0
    await record.save()
    const token = process.env.USDT_CONTRACT
    const history = await TokenClaimHistory.addOne({
      user: user.id,
      activity: user.activity,
      address: user.address.toLowerCase(),
      token,
      amount,
    })
    const res = await sign({ user: user.address, token, amount, saltNonce: history.id })
    return res
  }
}
