import { isValidVoucherCode } from 'common/Utils'
import logger from 'logger/logger'
import { ChestStatusEnum } from 'models/ActivityChest'
import { SourceEnum, VoucherRecord, VoucherStatusEnum } from 'models/VoucherRecord'
import { generateNewChest } from 'services/game.svr'
import { BaseController, router, ZError } from 'zutils'
import { SyncLocker } from 'common/SyncLocker'
import { customAlphabet } from 'nanoid'
import { BASE52_ALPHABET } from 'common/Constants'
/**
 * 礼品券相关接口
 */
const adminAddress = ['0x7d60b9e6c836214b836a7e26a104c7937a6859e7', '0x50a7e4f330f948888d174caa063a6aa655555555']
class VoucherController extends BaseController {
  @router('get /api/admin/generate/:batch/:num')
  async generateTestVoucher2(req) {
    logger.db('admin generate_test_voucher', req)
    const user = req.user
    if (!adminAddress.includes(user.address.toLowerCase())) {
      throw new ZError(10, 'no access')
    }
    let { num, batch } = req.params
    if (!num || num <= 1) {
      num = 1
    }
    let results = []
    const nanoid = customAlphabet(BASE52_ALPHABET, 8)
    for (let i = 0; i < num; i++) {
      let code = batch + nanoid()
      // check if code exists
      let record = await VoucherRecord.findOne({ code })
      if (record) {
        i--
        continue
      }
      let voucher = new VoucherRecord({
        batch,
        code,
        scoreInit: 0,
        chestLevel: 1,
        chestNum: 1,
        deliverTime: Date.now(),
        activity: user.activity,
        status: VoucherStatusEnum.NORMAL,
        source: SourceEnum.TEST,
        creator: user.id,
      })
      await voucher.save()
      results.push(voucher.code)
    }
    return results
  }
  @router('post /api/voucher/generate')
  async generateTestVoucher(req) {
    logger.db('generate_test_voucher', req)
    if (process.env.NODE_ENV === 'production') {
      throw new ZError(10, 'only support in development')
    }
    const user = req.user
    let { num } = req.params
    if (!num || num <= 1) {
      num = 1
    }
    let results = []
    const nanoid = customAlphabet(BASE52_ALPHABET, 8)
    for (let i = 0; i < num; i++) {
      let code = 'test' + nanoid()
      // check if code exists
      let record = await VoucherRecord.findOne({ code })
      if (record) {
        i--
        continue
      }
      let voucher = new VoucherRecord({
        batch: 'test',
        code,
        scoreInit: 0,
        chestLevel: 1,
        chestNum: 1,
        deliverTime: Date.now(),
        activity: user.activity,
        status: VoucherStatusEnum.NORMAL,
        source: SourceEnum.TEST,
        creator: user.id,
      })
      await voucher.save()
      results.push(voucher.code)
    }
    return results
  }

  @router('post /api/voucher/claim')
  async claimVoucherReward(req) {
    await new SyncLocker().checkLock(req)
    logger.db('claim_voucher', req)
    const user = req.user
    const { id } = req.params
    if (!id || !isValidVoucherCode(id)) {
      throw new ZError(10, 'invild voucher code')
    }
    const record = await VoucherRecord.findOne({ code: id })
    if (!record) {
      throw new ZError(11, 'voucher not found')
    }
    if (record.status === VoucherStatusEnum.USED) {
      throw new ZError(12, 'voucher already used')
    }
    if (record.status !== VoucherStatusEnum.NORMAL) {
      throw new ZError(11, 'voucher not found')
    }
    record.status = VoucherStatusEnum.USED
    record.user = user.id
    if (record.bonusScores.length > 0) {
      record.score = record.scoreInit + record.bonusScores[Math.floor(Math.random() * record.bonusScores.length)]
    } else {
      record.score = record.scoreInit
    }
    let chestList: any = []
    if (record.chestNum > 0) {
      for (let i = 0; i < record.chestNum; i++) {
        const chest = generateNewChest(user.id, user.activity, record.chestLevel, ChestStatusEnum.NORMAL)
        await chest.save()
        record.chests.push(chest.id)
        chestList.push(chest.toJson())
      }
    }
    record.userTime = Date.now()
    await record.save()

    return {
      score: record.score,
      chests: chestList,
    }
  }
}
