import { Severity, getModelForClass, index, modelOptions, mongoose, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'

/**
 * nft hodler claim record
 */

@dbconn()
@index({ user: 1 }, { unique: false })
@index({ user: 1, group: 1 }, { unique: false })
@index({ user: 1, contract: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'nft_holder_claim_record', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
class NFTHolderRecordClass extends BaseModule {
  @prop({ required: true })
  public user: string
  @prop()
  public chain: number
  @prop({ required: true })
  public contract: string
  @prop()
  public group?: string
  @prop({ type: () => [String], default: [] })
  public tokenId?: string[]
  @prop()
  public holderNum: number
  @prop({ type: () => [String], default: [] })
  public rewards: string[]
}

export const NFTHolderRecord = getModelForClass(NFTHolderRecordClass, {
  existingConnection: NFTHolderRecordClass['db'],
})
