import { Severity, getModelForClass, index, modelOptions, mongoose, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'

export const DAILY_SIGN = 'daily_sign'
// 累计签到奖励
export const SIGN_TOTAL = 'sign_total'
// 连续签到奖励
export const SIGN_SEQ = 'sign_seq'
// 使用门票
export const USE_TICKET = 'use_ticket'
// 基础任务完成奖励
export const BASE_TASK_REWARD = 'base_task_reward'

@dbconn()
@index({ user: 1 }, { unique: false })
@index({ activity: 1 }, { unique: false })
@index({ user: 1, activity: 1, type: 1 }, { unique: false })
@modelOptions({
  schemaOptions: { collection: 'ticket_record', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
class TicketRecordClass extends BaseModule {
  @prop({ required: true })
  public user: string

  @prop({ required: true })
  public activity: string

  @prop()
  public score: number

  @prop()
  public type: string

  @prop({ type: mongoose.Schema.Types.Mixed })
  public data: any

  public toJson() {
    return {
      user: this.user,
      activity: this.activity,
      score: this.score,
      type: this.type,
      //@ts-ignore
      time: this.createdAt.getTime(),
    }
  }
}

export const TicketRecord = getModelForClass(TicketRecordClass, { existingConnection: TicketRecordClass['db'] })
