import { dbconn } from 'decorators/dbconn'
import { getModelForClass, index, modelOptions, mongoose, pre, prop } from '@typegoose/typegoose'
import { Severity } from '@typegoose/typegoose/lib/internal/constants'
import { BaseModule } from './Base'
import { convert } from 'zutils/utils/number.util'
const alphabet = '3fBCM8j17XNA9xYun4wmLWep2oHFlhPcgyEJskqOz6GK0UtV5ZRaDSvrTbidQI'

export enum VoucherStatusEnum {
  LOCKED = 0,
  INITED = 1,
  NORMAL = 2,
  USED = 9,
}

export enum SourceEnum {
  TEST = 'test',
  ADMIN = 'admin',
}
/**
 * 宝箱券
 */
@dbconn()
@index({ user: 1, activity: 1 }, { unique: false })
@index({ code: 1, activity: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'voucher_record', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
export class VoucherRecordClass extends BaseModule {
  @prop()
  public batch: string
  @prop()
  public code: string
  // 0 锁定, 1 未发放 2 已发放,待使用 9 已兑换
  @prop({ enum: VoucherStatusEnum, default: VoucherStatusEnum.INITED })
  public status: VoucherStatusEnum
  @prop()
  public user: string
  @prop()
  public activity: string
  // 基础积分
  @prop({ default: 0 })
  public scoreInit: number
  // 浮动积分
  @prop({ type: () => [Number], default: [] })
  public bonusScores: number[]
  // 可获得的宝箱等级
  @prop({ default: 1 })
  public chestLevel: number
  // 可获得的宝箱数量
  @prop({ default: 1 })
  public chestNum: number
  // 最终得到的积分
  @prop({ default: 0 })
  public score: number
  // 最终得到的宝箱列表
  @prop({ type: () => [String], default: [] })
  public chests: string[]
  @prop()
  public comment: string
  @prop()
  public creator: string
  @prop({ enum: SourceEnum, default: SourceEnum.ADMIN })
  public source: SourceEnum
  // 发放时间
  @prop()
  public deliverTime: number
  // 使用时间
  @prop()
  public userTime: number

  public toJson() {
    return {
      // @ts-ignore
      id: this.id,
      stat: this.status,
      code: this.code,
      score: this.score,
      chests: this.chests,
    }
  }
}
export const VoucherRecord = getModelForClass(VoucherRecordClass, { existingConnection: VoucherRecordClass['db'] })
