import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from '../Base'
import { hexToUtf8 } from 'zutils/utils/string.util'
import logger from 'logger/logger'

@dbconn('chain')
@index({ from: 1 }, { unique: false })
@index({ from: 1, chestId: 1 }, { unique: true })
@index({ from: 1, blockTime: 1 }, { unique: false })
@modelOptions({
  schemaOptions: { collection: 'chest_open_record', timestamps: true },
})
export class ChestRecordClass extends BaseModule {
  @prop({ required: true })
  public from!: string
  @prop()
  public to: string
  @prop({ required: true })
  public hash: string
  @prop()
  public blockNumber: string
  @prop()
  public blockHash: string
  @prop()
  public blockTime: number
  @prop()
  public dateTag: string
  @prop()
  public chestId: string
  @prop()
  public value: string
  @prop()
  public input: string

  public static async saveEvent(event: any) {
    const dataStr = hexToUtf8(event.input)
    const regexp = /data:,{\"p\":\"cf-20\",\"op\":\"chest_open\",\"id\":\"(.+?)\"}/
    const match = dataStr.match(regexp)
    if (!match) {
      logger.log('not a chest open event:', event.hash)
      return
    }
    event.chestId = match[1]
    logger.log('open chest with id:', event.chestId)
    return ChestRecord.insertOrUpdate({ hash: event.hash }, event)
  }

  public toJson() {
    return {
      address: this.from,
      day: this.dateTag,
      time: this.blockTime,
    }
  }
}

export const ChestRecord = getModelForClass(ChestRecordClass, {
  existingConnection: ChestRecordClass['db'],
})
