import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from '../Base'
import { ZERO_ADDRESS } from 'common/Constants'

@dbconn('chain')
@index({ chain: 1, address: 1, tokenId: 1 }, { unique: true })
@index({ chain: 1, address: 1, user: 1 }, { unique: false })
@modelOptions({
  schemaOptions: { collection: 'nft_holder', timestamps: true },
})
export class NftHolderClass extends BaseModule {
  @prop({ required: true })
  public address!: string
  @prop({ required: true })
  public chain: string
  @prop({ required: true })
  public tokenId: string
  @prop()
  public blockNumber: number
  @prop()
  public user: string
  @prop({ default: false })
  public burn: boolean

  public static async saveData(event: any) {
    const address = event.address
    const chain = event.chain
    const tokenId = event.tokenId
    const blockNumer = event.blockNumber
    const burn = event.to === ZERO_ADDRESS

    let record = await NftHolder.findOne({ address, chain, tokenId })
    if (!record) {
      record = new NftHolder({ address, chain, tokenId, blockNumber: blockNumer, user: event.to, burn })
    } else {
      if (record.blockNumber < blockNumer) {
        if (burn) {
          record.burn = true
        } else {
          record.user = event.to
        }
        record.blockNumber = blockNumer
      }
    }
    await record.save()
  }
}

export const NftHolder = getModelForClass(NftHolderClass, {
  existingConnection: NftHolderClass['db'],
})
