import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from '../Base'

@dbconn('chain')
@index({ chain: 1, nft: 1, tokenId: 1, start: 1 }, { unique: true })
@index({ chain: 1, user: 1, nft: 1 }, { unique: false })
@modelOptions({
  schemaOptions: { collection: 'nft_stake_info', timestamps: true },
})
export class NftStakeClass extends BaseModule {
  @prop({ required: true })
  public chain: string
  @prop()
  public blockNumber: number
  // stake info
  @prop()
  public address: string
  @prop()
  public user: string
  @prop()
  public nft: string
  @prop()
  public tokenId: string
  @prop()
  public start: number
  @prop()
  public stakeTime: number
  // 1: staked, 2: redeemed
  @prop()
  public status: number

  @prop()
  public redeemTime: number

  @prop({ default: 0 })
  public version: number

  public toJson() {
    return {
      nft: this.nft,
      nftid: this.tokenId,
      start: this.start,
      cd: +process.env.STAKE_CD,
      stakeTime: this.stakeTime,
    }
  }
}

export const NftStake = getModelForClass(NftStakeClass, {
  existingConnection: NftStakeClass['db'],
})
