import { getModelForClass, index, modelOptions, mongoose, prop, ReturnModelType, Severity } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from '../Base'

export enum PlatEnum {
  GOOGLE = 0,
  APPLE = 1,
  TIKTOK = 2,
  FACEBOOK = 3,
  TWITTER = 4,
  TELEGRAM = 5,
  EMAIL = 6,
  DISCORD = 7,
}

@dbconn('oauth')
@modelOptions({
  schemaOptions: { collection: 'auth_record', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
export class AuthRecordClass extends BaseModule {
  @prop({ required: true })
  public address: string

  @prop({ enum: PlatEnum, default: PlatEnum.DISCORD })
  public platform: PlatEnum

  @prop()
  public nickname?: string

  @prop()
  public username?: string
  @prop()
  public avatar?: string
  // 对应discord上的discriminator
  @prop()
  public discriminator?: string

  @prop()
  public openId?: string

  @prop()
  public email?: string

  @prop({ default: 0 })
  public condition: number

  @prop({ default: 0 })
  public version: number

  @prop()
  public tokenType: string
  @prop()
  public accessToken: string
  @prop()
  public refreshToken: string
  @prop()
  public expiresIn: number
  @prop()
  public scope: string

  @prop({ type: mongoose.Schema.Types.Mixed })
  public outData: any

  public static async findByAddress(
    this: ReturnModelType<typeof AuthRecordClass>,
    address: string,
    platform: PlatEnum,
  ) {
    return this.findOne({ address, platform }).exec()
  }
}

export const AuthRecord = getModelForClass(AuthRecordClass, { existingConnection: AuthRecordClass.db })
