import logger from 'logger/logger'
import { UserLog } from 'models/UserLog'
import { AsyncQueue, createAsyncQueue, singleton } from 'zutils'

@singleton
export class LoggerQueue {
  private queue: AsyncQueue

  constructor() {
    this.queue = createAsyncQueue()
  }

  public addLog(name, req, logObj) {
    this.queue.push(async () => {
      const user = req.user
      const ip = req.headers['x-forwarded-for'] || req.ip
      const path = req.url
      const params = req.method === 'GET' ? req.query : req.body
      const dataObj = JSON.stringify(logObj) === '{}' ? params : logObj
      try {
        const history = new UserLog({
          user: user ? user.id : '',
          path: path,
          method: req.method,
          params: dataObj,
          referer: req.headers['referer'],
          user_agent: req.headers['user-agent'],
          ip,
          name,
        })
        await history.save()
      } catch (err) {
        logger.error('error add user log: ')
        logger.error(err)
      }
    })
  }
}
