import { INVITE_REBATE, INVITE_REBATE_LEVEL, RANK_SCORE_SCALE, SCORE_INVITE_REBATE } from 'common/Constants'
import { ActivityUser } from 'models/ActivityUser'
import { ScoreRecord } from 'models/ScoreRecord'
import { ZRedisClient } from 'zutils'
import { formatDate } from 'utils/utcdate.util'
const rankLevels = require('../../configs/uaw_rank_level.json')
/**
 * 更新排行榜
 * @param param0
 *          user: 用户id
 *          score: 分数
 *          activity: 活动id
 *          scoreType: 分数类型
 *          scoreParams: 额外的参数
 *          rebatLevel: 返利层级
 */
export const updateRankScore = async ({
  user,
  score,
  activity,
  scoreType,
  scoreParams,
  rebateLevel = INVITE_REBATE_LEVEL,
}: {
  user: string
  score: number
  activity: string
  scoreType: string
  scoreParams: any
  rebateLevel?: number
}) => {
  let record = new ScoreRecord({
    user: user,
    activity: activity,
    score,
    type: scoreType,
    data: scoreParams,
  })
  await record.save()
  const key = rankKey(activity)
  await updateRank(key, score, user)
  // add daily score
  const dailyKey = rankKey(activity, new Date())
  await updateRank(dailyKey, score, user)
  // 给邀请人返利
  if (rebateLevel > 0) {
    let userMod = await ActivityUser.findById(user)
    if (userMod?.inviteUser) {
      const score1 = score * INVITE_REBATE
      let scoreParams1 = Object.assign({ fromUser: user }, scoreParams)
      const keyInvite = `${activity}:invite`
      await updateRankInvite(keyInvite, score1, `${userMod.inviteUser}_${user}`)
      await updateRankScore({
        user: userMod.inviteUser,
        score: score1,
        activity,
        scoreType: SCORE_INVITE_REBATE,
        scoreParams: scoreParams1,
        rebateLevel: rebateLevel - 1,
      })
    }
  }
}

/**
 * 更新排行榜
 * @param key
 * @param score
 * @param member
 */
export const updateRank = async (key: string, score: number, member: string) => {
  score = parseInt(score * RANK_SCORE_SCALE + '')
  await new ZRedisClient().zincrby(key, score, member)
}

export const updateRankInvite = async (key: string, score: number, member: string) => {
  score = parseInt(score * RANK_SCORE_SCALE + '')
  await new ZRedisClient().zincrby(key, score, member)
}

export const rankKey = (activity: string, date?: Date) => {
  if (!date) {
    return `${activity}:score`
  }
  const dateTag = formatDate(date)
  return `${activity}:score:${dateTag}`
}

export const rankLevel = (rank: number | string) => {
  rank = parseInt(rank + '')
  const data = rankLevels.find(o => rank >= o.rankMin && rank <= o.rankMax)
  return data ? data.level : '-'
}
