import { ITask } from './base/ITask'
import { TaskStatusEnum } from 'models/ActivityUser'
import { ZError } from 'zutils'
import { TaskCfg } from 'models/ActivityInfo'
import { queryBurnNftList } from 'services/chain.svr'
import { NftBurnRecord } from 'models/NFTBrunRecord'

export default class BurnNft extends ITask {
  static desc = 'Butn NFT'
  static show: boolean = true
  async execute(data: any) {
    const { task } = data
    let cfg = this.activity.tasks.find((t: TaskCfg) => t.id === task.id)
    const address = cfg.params.address.toLowerCase()
    const chain = process.env.CHAIN
    const nftList = await queryBurnNftList(address, this.user.address, chain)
    const localNft = await NftBurnRecord.find({
      user: this.user.id,
      chain,
      address,
    })
    const localNftSet = new Set()
    let finishAmount = 0
    let tmpNftSet = new Set()
    for (let nft of localNft) {
      localNftSet.add(nft.tokenId)
      if (nft.activity === this.activity.id && nft.task === task.id) {
        finishAmount += 1
        tmpNftSet.add(nft.tokenId)
      }
    }
    let finishNfts = new Set()
    for (let nft of nftList) {
      if (!localNftSet.has(nft.tokenId)) {
        finishNfts.add(nft.tokenId)
        finishAmount += 1
        if (finishAmount >= cfg.repeat) {
          break
        }
      }
    }
    for (let finishNft of finishNfts) {
      const record = new NftBurnRecord({
        user: this.user.id,
        chain,
        address,
        tokenId: finishNft,
        activity: this.activity.id,
        task: task.id,
      })
      await record.save()
    }
    let dataToSave = task.data || {}
    dataToSave.tokenId = [...tmpNftSet, ...finishNfts]
    dataToSave.finishAmount = finishAmount
    task.data = dataToSave
    task.markModified('data')
    if (finishAmount >= cfg.repeat) {
      task.status = TaskStatusEnum.SUCCESS
      task.timeFinish = Date.now()
    } else if (finishAmount > 0) {
      task.status = TaskStatusEnum.PART_SUCCESS
    }
    try {
      await this.user.save()
    } catch (err) {
      throw new ZError(100, 'save failed')
    }
    if (cfg.autoclaim) {
      try {
        await this.claimReward(task)
      } catch (err) {
        console.log(err)
      }
    }
    return true
  }
}
