import { checkTwitter } from 'services/oauth.svr'
import { ITask } from './base/ITask'
import { TaskStatusEnum } from 'models/ActivityUser'
import { ZError } from 'zutils'
import { TaskCfg } from 'models/ActivityInfo'

export default class TwitterConnect extends ITask {
  static desc = 'twitter connect'
  static show: boolean = true

  async execute(data: any) {
    let { address } = this.user
    let res = await checkTwitter(address)
    if (res.errcode) {
      throw new ZError(res.errcode, res.errmsg)
    }
    const { task } = data
    if (res.data.userid && task.status === TaskStatusEnum.RUNNING) {
      task.status = TaskStatusEnum.SUCCESS
      task.timeFinish = Date.now()
      task.data = res.data
      this.user.twitterId = res.data.userid
      this.user.twitterName = res.data.username
      this.user.twitterAvatar = res.data.avatar
      try {
        await this.user.save()
      } catch (err) {
        throw new ZError(100, 'twitter already binded')
      }
      let cfg = this.activity.tasks.find((t: TaskCfg) => t.id === task.id)
      if (cfg.autoclaim) {
        try {
          await this.claimReward(task)
        } catch (err) {
          console.log(err)
        }
      }
    }
    return true
  }
}
