import { ActivityInfoClass, TaskCfg } from 'models/ActivityInfo'
import { ActivityItem } from 'models/ActivityItem'
import { ActivityUserClass, TaskStatusEnum } from 'models/ActivityUser'
import { updateRankScore } from 'services/rank.svr'
import { DocumentType } from '@typegoose/typegoose'

export abstract class ITask {
  static desc: string
  static show: boolean = true
  static auto: boolean = false

  user: DocumentType<ActivityUserClass>
  activity: DocumentType<ActivityInfoClass>

  constructor({
    user,
    activity,
  }: {
    user: DocumentType<ActivityUserClass>
    activity: DocumentType<ActivityInfoClass>
  }) {
    // do nothing
    this.user = user
    this.activity = activity
  }
  abstract execute(data: any): Promise<boolean>

  public async claimReward(task: any) {
    const user = this.user
    const [taskId, dateTag] = task.id.split(':')
    const cfg = this.activity.tasks.find((t: TaskCfg) => t.id === taskId)
    if (!cfg.score) {
      return
    }
    let claimAmount = task.data?.claimAmount || 0
    let score = cfg.score
    if (user.boost > 1 && Date.now() < user.boostExpire.getTime()) {
      score = Math.floor(score * user.boost)
    }
    cfg.repeat = cfg.repeat || 1
    let finishAmount = cfg.repeat > 1 ? task.data?.finishAmount || 0 : 1
    for (let i = claimAmount; i < Math.min(cfg.repeat, finishAmount); i++) {
      await updateRankScore({
        user: user.id,
        score: score,
        activity: user.activity,
        scoreType: cfg.category,
        scoreParams: {
          date: dateTag,
          taskId: task.id,
          boost: user.boost,
        },
      })
      claimAmount += 1
    }
    task.data = Object.assign(task.data || {}, { claimAmount })
    task.markModified('data')
    if ((cfg.repeat > 1 && claimAmount >= cfg.repeat) || (cfg.repeat === 1 && claimAmount >= 1)) {
      task.status = TaskStatusEnum.CLAIMED
      task.timeClaim = Date.now()
    }
    await this.claimItem(cfg)
    await user.save()
    return { score }
  }

  public async claimItem(cfg: TaskCfg) {
    if (cfg.params.items) {
      for (let key in cfg.params.items) {
        let amount = cfg.params.items[key]
        await ActivityItem.insertOrUpdate(
          { user: this.user.id, activity: this.user.activity, item: key },
          { $inc: { amount }, last: Date.now() },
        )
      }
    }
  }
}
